@@ -31,11 +31,13 @@ public:
 public slots:
 	virtual void accept();
 signals:
-	void connectToHost(QString url, int quality);
+	void connectToHost(QString url, int quality, int listen_port);
 private slots:
+	void indexChanged(int index);
 	void hostnameUpdated(QString newtext);
 private:
 	QHBoxLayout layout;
+	QStringList hostnames_sorted;
 	QComboBox hosts;
 	QMaemo5ListPickSelector *quality_selector;
 };
