@@ -41,13 +41,12 @@ class VncView: public RemoteView
     Q_OBJECT
 
 public:
-    explicit VncView(QWidget *parent = 0, const KUrl &url = KUrl(), RemoteView::Quality quality = RemoteView::Medium);
+    explicit VncView(QWidget *parent = 0, const KUrl &url = KUrl(), RemoteView::Quality quality = RemoteView::Medium, int listen_port = 0);
     ~VncView();
 
     QSize framebufferSize();
     QSize sizeHint() const;
     QSize minimumSizeHint() const;
-    void startQuitting();
     bool isQuitting();
     void reloadSettings();
     bool start();
@@ -59,10 +58,10 @@ public:
     void showDotCursor(DotCursorState state);
     
 public slots:
-    void enableScaling(bool scale);
-    void scaleResize(int w, int h);
+    void setZoomLevel(int level = -1); //'level' doesn't correspond to actual magnification, though mapping is done here
     void sendKey(Qt::Key key);
     void sendKeySequence(QKeySequence keys);
+    void startQuitting();
     void forceFullRepaint();
 
 protected:
@@ -89,9 +88,10 @@ private:
     QImage m_frame;
     bool m_forceLocalCursor;
     bool force_full_repaint;
-    RemoteView::Quality quality;
     int left_zoom, right_zoom;
     bool disable_tapping;
+    RemoteView::Quality quality;
+	int listen_port;
 
     void keyEventHandler(QKeyEvent *e);
     void unpressModifiers();
